"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var Source_1 = require("./Source");
var Inference_1 = require("./Inference");
var CombineClasses_1 = require("./CombineClasses");
var JSONSchemaInput_1 = require("./JSONSchemaInput");
var TypeBuilder_1 = require("./TypeBuilder");
var InferMaps_1 = require("./InferMaps");
var TargetLanguage = /** @class */ (function () {
    function TargetLanguage(displayName, names, extension, optionDefinitions) {
        this.displayName = displayName;
        this.names = names;
        this.extension = extension;
        this.optionDefinitions = optionDefinitions;
    }
    TargetLanguage.prototype.transformAndRenderConfig = function (config) {
        var typeBuilder = new TypeBuilder_1.TypeGraphBuilder();
        var combine = config.combineClasses;
        if (config.isInputJSONSchema) {
            for (var _i = 0, _a = config.topLevels; _i < _a.length; _i++) {
                var tlc = _a[_i];
                typeBuilder.addTopLevel(tlc.name, JSONSchemaInput_1.schemaToType(typeBuilder, tlc.name, tlc.schema));
            }
            combine = false;
        }
        else {
            var inference_1 = new Inference_1.TypeInference(typeBuilder, config.inferMaps, this.supportsEnums && config.inferEnums);
            config.topLevels.forEach(function (tlc) {
                typeBuilder.addTopLevel(tlc.name, inference_1.inferType(config.compressedJSON, tlc.name, false, tlc.samples));
            });
        }
        var graph = typeBuilder.finish();
        if (!config.isInputJSONSchema) {
            if (combine) {
                graph = CombineClasses_1.combineClasses(graph);
            }
            if (config.inferMaps) {
                graph = InferMaps_1.inferMaps(graph);
            }
        }
        if (!config.doRender) {
            return { lines: ["Done.", ""], annotations: immutable_1.List() };
        }
        var renderResult = this.renderGraph(graph, config.rendererOptions);
        return Source_1.serializeRenderResult(renderResult, this.indentation);
    };
    TargetLanguage.prototype.needsCompressedJSONInput = function (rendererOptions) {
        return true;
    };
    Object.defineProperty(TargetLanguage.prototype, "indentation", {
        get: function () {
            return "    ";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TargetLanguage.prototype, "supportsEnums", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return TargetLanguage;
}());
exports.TargetLanguage = TargetLanguage;
