"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var Type_1 = require("./Type");
var Support_1 = require("./Support");
var TypeBuilder_1 = require("./TypeBuilder");
var TypeGraph = /** @class */ (function () {
    function TypeGraph() {
        var _this = this;
        this._frozen = false;
        // FIXME: OrderedMap?  We lose the order in PureScript right now, though,
        // and maybe even earlier in the TypeScript driver.
        this._topLevels = immutable_1.Map();
        this._types = immutable_1.List();
        this.freeze = function (topLevels, types) {
            Support_1.assert(!_this._frozen, "Tried to freeze TypeGraph a second time");
            Support_1.assert(types.every(function (t) { return t.typeRef.graph === _this; }), "Trying to freeze a graph with types that don't belong in it");
            _this._frozen = true;
            _this._topLevels = topLevels;
            _this._types = types;
        };
        this.typeAtIndex = function (index) {
            Support_1.assert(_this._frozen, "Cannot get type from a non-frozen graph");
            return Support_1.defined(Support_1.defined(_this._types).get(index));
        };
        this.allNamedTypes = function (childrenOfType) {
            return _this.filterTypes(function (t) { return t.isNamedType(); }, childrenOfType);
        };
        this.allNamedTypesSeparated = function (childrenOfType) {
            var types = _this.allNamedTypes(childrenOfType);
            return Type_1.separateNamedTypes(types);
        };
        // Each array in `replacementGroups` is a bunch of types to be replaced by a
        // single new type.  `replacer` is a function that takes a group and a
        // TypeBuilder, and builds a new type with that builder that replaces the group.
        // That particular TypeBuilder will have to take as inputs types in the old
        // graph, but return types in the new graph.  Recursive types must be handled
        // carefully.
        this.rewrite = function (replacementGroups, replacer) {
            return new TypeBuilder_1.GraphRewriteBuilder(_this, replacementGroups, replacer).finish();
        };
    }
    Object.defineProperty(TypeGraph.prototype, "topLevels", {
        get: function () {
            Support_1.assert(this._frozen, "Cannot get top-levels from a non-frozen graph");
            return Support_1.defined(this._topLevels).map(function (tref) { return tref.deref(); });
        },
        enumerable: true,
        configurable: true
    });
    TypeGraph.prototype.filterTypes = function (predicate, childrenOfType) {
        var seen = immutable_1.Set();
        var types = immutable_1.List();
        function addFromType(t) {
            if (seen.has(t))
                return;
            seen = seen.add(t);
            var children = childrenOfType ? childrenOfType(t) : t.children;
            children.forEach(addFromType);
            if (predicate(t)) {
                types = types.push(t);
            }
        }
        this.topLevels.forEach(addFromType);
        return types.reverse().toOrderedSet();
    };
    return TypeGraph;
}());
exports.TypeGraph = TypeGraph;
